!
!     READ INPUTFILE
!
SUBROUTINE readinit

use globalmod

implicit none
! *****local variables
integer i
real*8 xmin,xmax,ymin,ymax,zmin,zmax
character line*1024,outline*1024,comp*5

! reading initial value and frequency
CALL read_a_line(25,line)
read (line,*) nskip,nread
if (nskip.lt.0) then
  write(6,'(a)') 'Wrong initial value'
  stop
end if
if (nread.lt.1) then
  write(6,'(a)') 'Wrong frequency'
  stop
end if
write(outline,*) '(init value,frequency)=(',nskip,',',nread,')'
write(6,'(a)') trim(outline)
! amount of charge transfer, msd calculations
CALL read_a_line(25,line)
read (line,*) logQ,logmsd
write(outline,*) 'amount of charge calculation: ',logQ,' msd calculation: ',logmsd
write(6,'(a)') trim(outline)
if (logQ) then
  ! reading PSF and DCD files
  CALL read_a_line(25,line)
  read (line,*) fname1,fname2,DCDUnitCell
  if (DCDunitcell) then
    write(outline,*) trim(fname1),' PSF file and ',trim(fname2),' DCD file. Unit cell is readed in DCD file'
  else
    write(outline,*) trim(fname1),' PSF file and ',trim(fname2),' DCD file'
  endif
  write(6,'(a)') trim(outline)
  ! reading time step size
  CALL read_a_line(25,line)
  read (line,*) dt
  if (dt.le.0.0d0) then
    write(6,'(a)') 'Wrong time step size'
    stop
  endif
  write(outline,*) dt,' time step size'
  write(6,'(a)') trim(outline)
  ! reading component along channel axis
  CALL read_a_line(25,line)
  read (line,*) comp
  if (trim(comp).eq.'x' .or. trim(comp).eq.'X') then
    write(6,*) 'x-component along channel axis'
    ic = 1
  else if  (trim(comp).eq.'y' .or. trim(comp).eq.'Y') then
    write(6,*) 'y-component along channel axis'
    ic = 2
  else if (trim(comp).eq.'z' .or. trim(comp).eq.'Z') then
    write(6,*) 'z-component along channel axis'
    ic = 3
  else
    write(6,'(a)') 'Wrong component along channel axis'
    stop
  endif
  ! reading box center and length in the three cartesian directions
  CALL read_a_line(25,line)
  read(line,*) xcen,ycen,zcen,Lx,Ly,Lz
  write(outline,*) Lx,' box lenght and ',xcen,' box centel along x-axis'
  write(6,'(a)') trim(outline)
  write(outline,*) Ly,' box lenght and ',ycen,' box centel along y-axis'
  write(6,'(a)') trim(outline)
  write(outline,*) Lz,' box lenght and ',zcen,' box centel along z-axis'
  write(6,'(a)') trim(outline)
  Lx2 = 0.5d0*Lx
  Ly2 = 0.5d0*Ly
  Lz2 = 0.5d0*Lz
  xmin = xcen - Lx2
  xmax = xcen + Lx2
  ymin = ycen - Ly2
  ymax = ycen + Ly2
  zmin = zcen - Lz2
  zmax = zcen + Lz2
  write(outline,*) 'Box dimension along x-axis: [',xmin,',',xmax,']'
  write(6,'(a)') trim(outline)
  write(outline,*) 'Box dimension along y-axis: [',ymin,',',ymax,']'
  write(6,'(a)') trim(outline)
  write(outline,*) 'Box dimension along z-axis: [',zmin,',',zmax,']'
  write(6,'(a)') trim(outline)
  ! reading planes which define the channel region
  CALL read_a_line(25,line)
  read (line,*) x1,x2,y1,y2,z1,z2
 if (x1.ge.x2) then
    write(6,'(a)') 'Wrong planes which define the channel region'
    stop
  endif
  if (x1.lt.xmin) then
    write(6,'(a)') 'Wrong minimum plane defining the channel region'
    stop
  endif
  if (x2.gt.xmax) then
    write(6,'(a)') 'Wrong maximum plane defining the channel region'
    stop
  endif
 if (y1.ge.y2) then
    write(6,'(a)') 'Wrong planes which define the channel region'
    stop
  endif
  if (y1.lt.ymin) then
    write(6,'(a)') 'Wrong minimum plane defining the channel region'
    stop
  endif
  if (y2.gt.ymax) then
    write(6,'(a)') 'Wrong maximum plane defining the channel region'
    stop
  endif
  if (z1.ge.z2) then
    write(6,'(a)') 'Wrong planes which define the channel region'
    stop
  endif
  if (z1.lt.zmin) then
    write(6,'(a)') 'Wrong minimum plane defining the channel region'
    stop 
  endif
  if (z2.gt.zmax) then
    write(6,'(a)') 'Wrong maximum plane defining the channel region'
    stop 
  endif
  write(outline,*) 'Planes which define the channel region along x-axis: [',x1,',',x2,']'
  write(6,'(a)') trim(outline)  
  write(outline,*) 'Planes which define the channel region along y-axis: [',y1,',',y2,']'
  write(6,'(a)') trim(outline)
  write(outline,*) 'Planes which define the channel region along z-axis: [',z1,',',z2,']'
  write(6,'(a)') trim(outline)
  iLcx = x2 - x1
  iLcy = y2 - y1
  iLcz = z2 - z1
  write(outline,*) 'Channel length along x-axis = ',iLcx
  write(6,'(a)') trim(outline)
  write(outline,*) 'Channel length along y-axis = ',iLcy
  write(6,'(a)') trim(outline)
  write(outline,*) 'Channel length along z-axis = ',iLcz
  write(6,'(a)') trim(outline)
  iLcx = 1.0/iLcx
  iLcy = 1.0/iLcy
  iLcx = 1.0/iLcx
  x1 = x1 - xmin
  x2 = x2 - xmin
  y1 = y1 - ymin
  y2 = y2 - ymin
  z1 = z1 - zmin
  z2 = z2 - zmin
  ! reading logical variable which indicates if all atoms are considered
  CALL read_a_line(25,line)
  read (line,*) logtot
  if (logtot) then
    write(6,*) 'Every atom is considered for computing amount of charge transfer'
    Ntypes = 0
  else
    write(6,*) 'Ions are only considered for computing amount of charge transfer'
    ! reading number of ion types
    CALL read_a_line(25,line)
    read (line,*) Ntypes
    if (Ntypes.le.0) then
      write(6,'(a)') 'Wrong ion types'
      stop
    endif
    write(outline,*) 'Number of ion types = ',Ntypes
    write(6,'(a)') trim(outline)
    allocate (typenam(Ntypes))
    ! reading ion names
    CALL read_a_line(25,line)
    read (line,*) (typenam(i),i=1,Ntypes)
    write(6,*) 'Ion names:'
    write(outline,*) (trim(typenam(i))//' ',i=1,Ntypes)
    write(6,'(a)') trim(outline)
  endif 
  ! reading outputfile name
  CALL read_a_line(25,line)
  read(line,*) Qout1
  if (Qout1) then
    CALL read_a_line(25,line)
    read (line,*) fname3
    write(outline,*) trim(fname3),' outputfile name for amount of charge transfer'
    write(6,'(a)') trim(outline)
  else
    if (.not.logmsd) then
      write(6,'(a)') 'Outpufile for amount of charge transfer is not activated'
      stop
    endif
  endif
endif
if (logmsd) then
  ! reading amount of charge transfer file
  if (.not.logQ) then
    CALL read_a_line(25,line)
    read(line,*) fname3
    write(outline,*) trim(fname3),' inputfile name for amount of charge transfer'
    write(6,'(a)') trim(outline)
  endif
  ! reading initial value and final value for elapsed time
  CALL read_a_line(25,line)
  read(line,*) initval,endval
  if (initval.lt.1) then
    write(6,'(a)') 'Wrong initial value for elapsed time'
    stop
  endif
  if (endval.lt.initval) then
    write(6,'(a)') 'Wrong final value for elapsed time'
    stop
  endif
  write(outline,*) initval,' initial value for elapsed time and ',endval,' final value for elapsed time'
  write(6,'(a)') trim(outline)
  ! reading outputfile name
  CALL read_a_line(25,line)
  read(line,*) Qout2
  if (Qout2) then
    CALL read_a_line(25,line)
    read (line,*) fname4
    write(outline,*) trim(fname4),' outputfile name for mean-square displacements'
    write(6,'(a)') trim(outline)
  endif
  ! reading regressions for obtaining self-diffusion constant
  CALL read_a_line(25,line)
  read(line,*) Qregr  
  if (Qregr) then
    CALL read_a_line(25,line)
    read(line,*) fname5
    if (endval.gt.(initval+1)) then
      write(outline,*) trim(fname5),' outputfile name for linear least square regressions'
      write(6,'(a)') trim(outline)
    else
      write(6,*) 'Linear least square regressions are turned off because number of time origins is lower than 3'
      Qregr = .false.
    endif
  endif
  if (.not.Qout2 .and. .not.Qregr) then 
    write(6,'(a)') 'Mean-square displacements and linear least square regressions are desactivated'
    stop
  endif
endif

return
end subroutine

SUBROUTINE read_a_line(iunit,line)

implicit none

! *****shared variables

integer iunit
character*1024 line
logical ok

ok = .true.
do while (ok)
  read(iunit,'(a144)',end=100) line
  ok = line(1:1) .eq. '*' .or. line(1:1) .eq. '#' .or. line(1:1) .eq. '!'
end do
return
100   continue
write (6,*) 'end of file ',iunit,' was reached'
stop

end subroutine

!
!     READ PSF FILE
!
subroutine psf

use globalmod

implicit none
integer :: dummyi, i, IDat, ilines, irest, ibonds
integer :: iat1, iat2, iat3, iat4, iat5, iat6, iat7, iat8
real*8 :: atomchar
character :: dummyc*144,atname*50,line*1024 

! read header
read(22,*)

! read title section
read(22,*)
read(22,*) dummyi,dummyc
do i = 1,dummyi
  read(22,*)
enddo

! read atoms section
read(22,*)
read(22,*) Nat,dummyc
allocate (atnam(Nat),atchar(Nat))
do i = 1,Nat
  read(22,*) IDat,dummyc,dummyi,dummyc,dummyc,atname,atomchar,dummyi
  if (IDat.le.0 .or. IDat.gt.Nat) then
    write(6,'(a)') 'Wrong atom ID in PSF file'
    stop
  endif
  atnam(IDat) = atname
  atchar(IDat) = atomchar
enddo

! read bond section
read(22,*)
read(22,*) nbonds,dummyc
allocate (bonds(2,nbonds))
ilines = nbonds/4 ! lines to read
ibonds = 0
do i = 1,ilines
  read(22,*) iat1,iat2,iat3,iat4,iat5,iat6,iat7,iat8 
  ! first bond
  if (iat1.le.0 .or. iat2.le.0 .or. iat1.eq.iat2) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat1.lt.iat2) then
    bonds(1,ibonds) = iat1
    bonds(2,ibonds) = iat2
  else
    bonds(1,ibonds) = iat2
    bonds(2,ibonds) = iat1
  endif
  ! second bond
  if (iat3.le.0 .or. iat4.le.0 .or. iat3.eq.iat4) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat3.lt.iat4) then
    bonds(1,ibonds) = iat3
    bonds(2,ibonds) = iat4
  else
    bonds(1,ibonds) = iat4
    bonds(2,ibonds) = iat3
  endif
  ! third bond
  if (iat5.le.0 .or. iat6.le.0 .or. iat5.eq.iat6) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat5.lt.iat6) then
    bonds(1,ibonds) = iat5
    bonds(2,ibonds) = iat6
  else
    bonds(1,ibonds) = iat6
    bonds(2,ibonds) = iat5
  endif
  ! fourth bond
  if (iat7.le.0 .or. iat8.le.0 .or. iat7.eq.iat8) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat7.lt.iat8) then
    bonds(1,ibonds) = iat7
    bonds(2,ibonds) = iat8
  else
    bonds(1,ibonds) = iat8
    bonds(2,ibonds) = iat7
  endif
enddo
irest = nbonds - ibonds ! rest of bonds to read in the last line
if (irest.eq.1) then
  read(22,*) iat1,iat2
  if (iat1.le.0 .or. iat2.le.0 .or. iat1.eq.iat2) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat1.lt.iat2) then
    bonds(1,ibonds) = iat1
    bonds(2,ibonds) = iat2
  else
    bonds(1,ibonds) = iat2
    bonds(2,ibonds) = iat1
  endif
else if (irest.eq.2) then
  read(22,*) iat1,iat2,iat3,iat4
  ! first bond
  if (iat1.le.0 .or. iat2.le.0 .or. iat1.eq.iat2) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat1.lt.iat2) then
    bonds(1,ibonds) = iat1
    bonds(2,ibonds) = iat2
  else
    bonds(1,ibonds) = iat2
    bonds(2,ibonds) = iat1
  endif
  ! second bond
  if (iat3.le.0 .or. iat4.le.0 .or. iat3.eq.iat4) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat3.lt.iat4) then
    bonds(1,ibonds) = iat3
    bonds(2,ibonds) = iat4
  else
    bonds(1,ibonds) = iat4
    bonds(2,ibonds) = iat3
  endif
else
  read(22,*) iat1,iat2,iat3,iat4,iat5,iat6
  ! first bond
  if (iat1.le.0 .or. iat2.le.0 .or. iat1.eq.iat2) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat1.lt.iat2) then
    bonds(1,ibonds) = iat1
    bonds(2,ibonds) = iat2
  else
    bonds(1,ibonds) = iat2
    bonds(2,ibonds) = iat1
  endif
  ! second bond
  if (iat3.le.0 .or. iat4.le.0 .or. iat3.eq.iat4) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat3.lt.iat4) then
    bonds(1,ibonds) = iat3
    bonds(2,ibonds) = iat4
  else
    bonds(1,ibonds) = iat4
    bonds(2,ibonds) = iat3
  endif
  ! third bond
  if (iat5.le.0 .or. iat6.le.0 .or. iat5.eq.iat6) then
    write(6,'(a)') 'Wrong bond atoms in PSF file'
    stop
  endif
  ibonds = ibonds + 1
  if (iat5.lt.iat6) then
    bonds(1,ibonds) = iat5
    bonds(2,ibonds) = iat6
  else
    bonds(1,ibonds) = iat6
    bonds(2,ibonds) = iat5
  endif
endif

! write information
write(6,'(1x,a,i6,a)') 'The PSF file contains ',Nat,' atoms'
write(6,*) 'ID atom---atom name---charge'
do i = 1,Nat
  write(line,*) i,' '//trim(atnam(i))//' ',atchar(i)
  write(6,'(a)') trim(line)
enddo
write(6,*) 'bond--atom1--atom2'
do i = 1,nbonds
  write(line,*) i,bonds(1,i),bonds(2,i)
  write(6,'(a)') trim(line)
enddo

return 
end subroutine

!
!     READ DCD FILE
!
subroutine dcd

use globalmod

implicit none
integer :: dummyi, nframes, initstep, ifreq, ntitle, natoms, i, j, iii, iat, jat, ibond, ipos
real :: timestep, d(6), aa, rboxgen(3), el2gen(3), tmp, dispmax, dispmin, xcenr(3)
real, allocatable :: xx(:,:), xreal(:,:), xxold(:,:), xrealold(:,:), xun(:,:)
character hdr*4, outline*144, line*1024

! initializations
rboxgen(1) = real(Lx)
rboxgen(2) = real(Ly)
rboxgen(3) = real(Lz)
el2gen(1) = real(Lx2)
el2gen(2) = real(Ly2)
el2gen(3) = real(Lz2)
xcenr(1) = real(xcen)
xcenr(2) = real(ycen)
xcenr(3) = real(zcen)
dispmax = 0.0
dispmin = 1.0e6

! read header 
read(23) hdr, nframes, initstep, ifreq, (dummyi,i=1,6), timestep, (dummyi,i=1,9)

! read title
read(23) ntitle,outline

! read total number of atoms
read(23) natoms
if (natoms.ne.Nat) then
  write(6,'(a)') 'Wrong number of atoms in DCD file'
  stop
endif

! allocations
if (nskip.ge.nframes) then
  write(6,'(a)') 'Wrong initial value in DCD file'
  stop
endif
nstep = (nframes-nskip)/nread ! Number of time steps
if (nstep.eq.0) then
  write(6,'(a)') 'Wrong frequency (and initial value) in DCD file'
  stop
endif
allocate (xx(3,Nat),xreal(3,Nat),xun(3,Nat),xxold(3,Nat),xrealold(3,Nat),xcoor(3,Nat,nstep))

! read coordinates
iii = 0
do i = 1, nframes
  if (DCDUnitCell) read(23) (d(j), j=1, 6)
  read(23) (xx(1,iat),iat=1,Nat)
  read(23) (xx(2,iat),iat=1,Nat)
  read(23) (xx(3,iat),iat=1,Nat)
  ! Translation of the cartesian coordinates
  do iat = 1,Nat
    do j = 1,3
      xx(j,iat) = xx(j,iat) - xcenr(j) + el2gen(j)
      if (xx(j,iat).lt.0.0 .or. xx(j,iat).gt.rboxgen(j)) then
        write(6,'(a)') 'Wrong box center and/or box length'
        stop
      endif
    end do
  end do
  if (i.gt.nskip) then
    ! Treatment of the Cartesian coordinates
    iii = iii + 1
    if (iii.eq.1) then
      do iat = 1,Nat
        do j = 1,3
          xreal(j,iat) = xx(j,iat)
        end do
      end do
    else
      do iat = 1,Nat
        tmp = 0.0
        do j = 1,3
          aa = xx(j,iat) - xxold(j,iat)
          if (abs(aa).gt.el2gen(j)) aa = aa - sign(rboxgen(j),aa)
          xreal(j,iat) = xrealold(j,iat) + aa
          tmp = tmp + aa*aa
        end do 
        if (tmp.gt.dispmax) dispmax = tmp
        if (tmp.lt.dispmin) dispmin = tmp
      end do 
    endif ! iii.eq.1 
    do iat = 1,Nat
      do j = 1,3
        xxold(j,iat) = xx(j,iat)
        xrealold(j,iat) = xreal(j,iat)
      end do  
    end do
    ! unwrap by bonds
    do iat = 1,Nat
      do j = 1,3
        xun(j,iat) = xreal(j,iat)
      end do
    end do   
    do ibond = 1,nbonds
      iat = bonds(1,ibond)
      jat = bonds(2,ibond)
      do j = 1, 3
        aa = xreal(j,jat) - xreal(j,iat)
        if (abs(aa) .gt. el2gen(j)) aa = aa - sign(rboxgen(j),aa)
        xun(j,jat) = xun(j,iat) + aa
      end do
    end do
    ! Obtention of cartesian coordinates along channel axis
    if (mod(iii,nread).eq.0) then
      ipos = iii/nread
      do iat = 1,Nat
        do j = 1,3
          xcoor(j,iat,ipos) = xun(j,iat)
        end do
      enddo
    endif ! mod(iii,nread).eq.0
  endif ! i.gt.nskip
end do ! next i 

! deallocations
deallocate (xx,xreal,xxold,xrealold,xun)

! write information
write(6,*) 'DCD FILE,'//hdr//':'
write(line,*) 'Number of frames  = ',nframes
write(6,'(a)') trim(line)
write(line,*) 'Initial time step = ',initstep
write(6,'(a)') trim(line)
write(line,*) 'Step interval = ',ifreq
write(6,'(a)') trim(line)
write(line,*) 'Time step size = ',timestep
write(6,'(a)') trim(line)
write(line,*) 'Total number of atoms = ',Nat
write(6,'(a)') trim(line)
write(line,*) 'Number of time steps = ',nstep
write(6,'(a)') trim(line)
write(line,*) 'Maximum displacement along normal direction to the channel',sqrt(dispmax)
write(6,'(a)') trim(line)
write(line,*) 'Minimum displacement along normal direction to the channel',sqrt(dispmin)
write(6,'(a)') trim(line)

return 
end subroutine

!    
! READ AMOUNT OF CHARGE TRANSFER FILE 
!      
subroutine Qread

use globalmod

implicit none
! *****local variables
integer i,iii,ipos
real*8 dispmaxr,dispmaxl,dispmaxc,timestep,timestepold,dtold
real*8 Qcharger,Qchargel,Qchargec,Qchargerold,Qchargelold,Qchargecold
real*8 aar,aal,aac,tmpr,tmpl,tmpc
character*1024 line
logical ok

! First reading: obtaining number of time steps
! read header
do i = 1,9
  read(21,*) 
enddo 

iii = 0 ! counter for amount of charge transfer file
ipos = 0 ! counter for amount of charge transfer along time
ok = .true.
ok = .true.
do while (ok)
  ! read data from the amount of charge transfer file
  read(21,*,iostat=i) timestep,Qcharger,Qchargel,Qchargec
  if (i.eq.0) then
    ok = .true.
    iii = iii + 1
  else if (i.gt.0) then
    write(6,'(a)') 'Error during read execution'
    stop
  else
    ok = .false. ! EOF
  endif
  if (ok .and. iii.gt.nskip) then
    ipos = ipos + 1
    if (mod(ipos,nread).eq.0) then
      nstep = ipos/nread 
    endif
  endif
enddo
if (nstep.lt.2) then
  write(6,'(a)') 'Only one time step'
  stop
endif

! allocations
allocate(Qr(nstep),Ql(nstep),Qc(nstep))

! Second reading: storing amount of charge transfer
! read header
do i = 1,9
  read(22,*) 
enddo 

dispmaxr = 0.0d0 ! maximum dislpacement (right-side difference operator)
dispmaxl = 0.0d0 ! maximum dislpacement (left-side difference operator)
dispmaxc = 0.0d0 ! maximum dislpacement (central difference operator)
iii = 0 ! counter for amount of charge transfer file
ipos = 0 ! counter for amount of charge transfer along time
ok = .true.
do while (ok)
  ! read data from the amount of charge transfer file
  read(22,*,iostat=i) timestep,Qcharger,Qchargel,Qchargec
  if (i.eq.0) then
    ok = .true.
    iii = iii + 1
  else if (i.gt.0) then
    write(6,'(a)') 'Error during read execution'
    stop
  else
    ok = .false. ! EOF
  endif
  if (ok .and. iii.gt.nskip) then
    ipos = ipos + 1
    if (ipos.gt.1) then
      aar = Qcharger - Qchargerold
      aal = Qchargel - Qchargelold
      aac = Qchargec - Qchargecold
      tmpr = dabs(aar)
      tmpl = dabs(aal)
      tmpc = dabs(aac)
      if (tmpr.gt.dispmaxr) dispmaxr = tmpr
      if (tmpl.gt.dispmaxl) dispmaxl = tmpl
      if (tmpc.gt.dispmaxc) dispmaxc = tmpc
      dt = timestep - timestepold
      if (ipos.gt.2) then
        if (abs(dt-dtold).gt.1.0d-12) then
          write(line,*) 'Different timestep sizes: ',dt,dtold
          write(6,'(a)') trim(line)
          stop
        endif
      endif
    endif
    Qchargerold = Qcharger
    Qchargelold = Qchargel
    Qchargecold = Qchargec
    timestepold = timestep
    if (ipos.gt.1) dtold = dt
    if (mod(ipos,nread).eq.0) then
      nstep = ipos/nread 
      Qr(nstep) = Qcharger
      Ql(nstep) = Qchargel
      Qc(nstep) = Qchargec
    endif
  endif
enddo
write(line,*) 'Timestep size :',dt
write(6,'(a)') trim(line) 
write(line,*) 'Maximum variation for collective charge transfer (right,left, central difference operators) :',dispmaxr,dispmaxl,dispmaxc
write(6,'(a)') trim(line)

return 
end subroutine

!
!     LINEAR LEAST SQUARE REGRESSION
!
SUBROUTINE linear_regression(npoints,x,y)

implicit none
integer, intent(in) :: npoints
real*8, intent(in), dimension(npoints) :: x,y
! *****local variables
integer :: j
real*8 :: a,b,r2,sea,seb
real*8 :: xav,yav,ssxx,ssyy,ssxy,s

! I) OBTENTION OF AVERAGE VALUES

xav = 0.0d0
yav = 0.0d0
do j = 1, npoints
  xav = xav + x(j)
  yav = yav + y(j)
enddo
xav = xav/npoints 
yav = yav/npoints 

! II) OBTENTION OF SUMS OF SQUARES

ssxx = 0.0d0
ssyy = 0.0d0
ssxy = 0.0d0
do j = 1, npoints
  ssxx = ssxx + x(j)*x(j)
  ssyy = ssyy + y(j)*y(j) 
  ssxy = ssxy + x(j)*y(j)
enddo
ssxx = ssxx - npoints*xav*xav
ssyy = ssyy - npoints*yav*yav   
ssxy = ssxy - npoints*xav*yav

! III) OBTENTION OF REGRESSION COEFFICIENT b AND a

b = ssxy/ssxx
a = yav - b*xav

! IV) OBTENTION OF CORRELATION COEFFICIENT r2 AND STANDARD ERRORS (sea,seb)

r2 = b*ssxy/ssyy
s = sqrt((ssyy-b*ssxy)/dble(npoints-2)) 
sea = s*sqrt(1.0d0/npoints+xav*xav/ssxx)
seb = s/sqrt(ssxx)

! V) WRITE INFORMATION         

write(91,*) 'Self-diffusion constant = ',b/2.0d0
write(91,*) 'Y-intercept = ',a
write(91,*) 'Correlation coefficient = ',r2
write(91,*) 'Standard errors (y-intercept,slope) = ',sea,seb

! END OF SUBROUTINE

return
end subroutine   

!    
! AMOUNT OF CHARGE TRANSFER 
!      
subroutine Qtransfer

use globalmod

implicit none
! *****local variables
integer n,i,k,j,Ncell(3)
integer, allocatable :: avat(:)
real*8 Qcharger,Qchargel,dQr,dQl,ei,xj(3),zjs,zji,initime,Qchr,Qchl,Qchc,L(3),iL(3),xmin(3),xmax(3)
logical ok,ok2
character*1024 line

! Initializations
L(1) = Lx
L(2) = Ly
L(3) = Lz
iL(1) = 1.0d0/Lx
iL(2) = 1.0d0/Ly
iL(3) = 1.0d0/Lz
xmin(1) = x1
xmin(2) = y1
xmin(3) = z1
xmax(1) = x2
xmax(2) = y2
xmax(3) = z2

! *** Reading input files
write(6,'(a)') '**** READING PSF AND DCD FILES ***'
open(22,file=trim(fname1),status="old")
open(23,file=trim(fname2),status="old",form='unformatted')
write(line,*) 'Opened ',trim(fname1),' PSF file and ',trim(fname2),' DCD file'
write(6,'(a)') trim(line)
call psf ! reading PSF file
call dcd ! reading DCD file
close(22)
close(23)
! *** Amount of charge transfer through the channel
write(6,'(a)') '*** CALCULATING AMOUNT OF CHARGE TRANSFER ***'
if (Qout1) then
  open(22,file=trim(fname3),status="unknown")
  write(line,*) 'Opened ',trim(fname3),' output file'
  write(6,'(a)') trim(line) 
  write(line,*) 'Number of particles inside the channel region:'
  write(6,'(a)') trim(line)
  write(line,*) 'simulation time',('---'//trim(typenam(k)),k=1,Ntypes)
  write(6,'(a)') trim(line)
  write(22,'(a)') '# Outputfile: Amount of charge transfer through the channel'
  write(line,*) '# Time steps = ',nstep
  write(22,'(a)') trim(line)
  write(line,*) '# Time step size = ',dt
  write(22,'(a)') trim(line)
  if (logtot) then
    write(line,*) '# Number of atoms = ',Nat
    write(22,'(a)') trim(line)
  else
    write(line,*) '# Number of ion types = ',Ntypes
    write(22,'(a)') trim(line)
  end if
  write(22,'(a)') '# time -> Simulation time'
  write(22,'(a)') '# Qr -> Amount of charge transfer using right-side difference operator for the component of the velocity'
  write(22,'(a)') '# Ql -> Amount of charge transfer using left-side difference operator for the component of the velocity'
  write(22,'(a)') '# Qc -> Amount of charge transfer using central difference operator for the component of the velocity'
  write(22,'(a)') '# time---Qr---Ql---Qc'
endif
if (logmsd) allocate(Qr(nstep-2),Ql(nstep-2),Qc(nstep-2))
allocate(avat(Ntypes))
initime = nskip*dt
Qcharger = 0.0d0
Qchargel = 0.0d0
do n = 2,nstep-1 ! sum over time steps
  dQr = 0.0
  dQl = 0.0 
  avat = 0
  do i = 1,Nat ! sum over atoms
    ei = atchar(i) ! charge of atom i
    ok = logtot
    k = 0
    do while (k.lt.Ntypes .and. .not.ok)
      k = k + 1
      ok = trim(atnam(i)).eq.trim(typenam(k))
    end do
    if (ok) then
      ok2 = .true.
      j = 0
      do while (j.lt.3 .and. ok2)
        j = j+1
        Ncell(j) = int(xcoor(j,i,n)*iL(j))
        xj(j) = xcoor(j,i,n) - Ncell(j)*L(j)
        ok2 = ok2 .and. xj(j).le.xmax(j).and.xj(j).ge.xmin(j)
      end do
      if (ok2) then
        avat(k) = avat(k) + 1
        zjs = xcoor(ic,i,n+1) - Ncell(ic)*L(ic)
        if (zjs.lt.xmin(ic)) zjs = xmin(ic)
        if (zjs.gt.xmax(ic)) zjs = xmax(ic)
        zji = xcoor(ic,i,n-1) - Ncell(ic)*L(ic)
        if (zji.lt.xmin(ic)) zji = xmin(ic)
        if (zji.gt.xmax(ic)) zji = xmax(ic)
        dQr = dQr + ei*(zjs-xj(ic))
        dQl = dQl + ei*(xj(ic)-zji)
      end if
    end if
  end do ! next i
  ! Amount of charge transfer (right-side difference operator)
  Qcharger = Qcharger + dQr
  Qchr = Qcharger*iL(ic)
  ! Amount of charge transfer (left-side difference operator)
  Qchargel = Qchargel + dQl 
  Qchl = Qchargel*iL(ic)
  ! Amount of charge transfer (central difference operator)
  Qchc = (Qcharger+Qchargel)*0.5d0*iL(ic)
  ! Write outputfile
  if (Qout1) then
    write(line,*) initime+n*nread*dt,Qchr,Qchl,Qchc
    write (22,'(a)') trim(line)
  endif
  if (logmsd) then
    Qr(n-1) = Qchr
    Ql(n-1) = Qchl
    Qc(n-1) = Qchc
  endif
  ! Write number of atoms in the selected region
  write(line,*) initime+n*nread*dt,(avat(k),k=1,Ntypes)
  write(6,'(a)') trim(line)
end do ! next n
nstep = nstep-2
close(22)
deallocate(avat)

return
end subroutine

!    
! MEAN-SQUARE DISPLACEMENT
!  
subroutine msd

use globalmod

implicit none
! *****local variables
integer norigin,npoints,npos,ik,itek
real*8 msddr,msddl,msddc,dkar,dkal,dkac,iorigin,ttime
real*8, allocatable :: elaptime(:),msdr(:),msdl(:),msdc(:)
character*1024 line

! *** reading amount of charge transfer 
if (.not.logQ) then
  write(6,'(a)') '**** READING AMOUNT OF CHARGE TRANSFER FILE ***'
  open(21,file=trim(fname3),status="old")
  open(22,file=trim(fname3),status="old")
  write(line,*) 'Opened ',trim(fname3),' amount of charge transfer file'
  write(6,'(a)') trim(line) 
  call Qread
  close(21)
  close(22)
else
  if (nstep.lt.2) then
    write(6,'(a)') 'Only one time step'
    stop
  endif 
endif

! *** Mean-square displacement 
write(6,*) '**** CALCULATING MEAN-SQUARE DISPLACEMENT FOR COLLECTIVE CHARGE ***'  
! Obtention of time origins
norigin = nstep/2
if (initval.gt.norigin) then
  write(6,'(a,i6)') 'Initial elapsed time value is greather than maximum value of ',norigin
  stop
endif
if (endval.gt.norigin) then
  write(6,'(a,i6)') 'Final elapsed time value is greather than maximum value of ',norigin
  write(6,'(a)') 'endval is modified to be equal to the number of origins'
  endval = norigin
endif
write(line,*) 'There are ',nstep,' times'
write(6,'(a)') trim(line)
write (line,*) 'There are ',norigin,' time origins'
write(6,'(a)') trim(line)
if (Qregr) then
  npoints = endval-initval+1    
  allocate (elaptime(npoints),msdr(npoints),msdl(npoints),msdc(npoints))
endif
iorigin = 1.0d0/dble(norigin)
! Open outputfile
if (Qout2) then
  open (88,file=trim(fname4),status='unknown')
  write(line,*) 'Opened ',trim(fname4),' output file'
  write(6,'(a)') trim(line) 
  write(88,'(a)') '# Outputfile: Mean-square displacement for the collective charge transfer'
  write(line,*) '# Time steps = ',nstep
  write(88,'(a)') trim(line)
  write(line,*) '# Time step size = ',dt
  write(88,'(a)') trim(line)
  write(88,'(a)') "# time -> elapsed time"
  write(88,'(a)') "# Qmsdr-> mean-square displacement for collective charge (right-side difference operator)"
  write(88,'(a)') "# Qmsdl-> mean-square displacement for collective charge (left-side difference operator)"
  write(88,'(a)') "# Qmsdc-> mean-square displacement for collective charge (central difference operator)"
  write(88,'(a)') "# time---Qmsdr---Qmsdl---Qmsdc"
endif

npoints =0
do npos = initval,endval ! npos is the elapsed time
  npoints = npoints + 1 
  ! Mean-square displacements
  msddr = 0.0d0
  msddl = 0.0d0
  msddc = 0.0d0 
  do ik = 1, norigin !ik is the starting position
    itek = ik + npos ! itek is the final position
    dkar = Qr(itek) - Qr(ik)
    dkal = Ql(itek) - Ql(ik)
    dkac = Qc(itek) - Qc(ik)
    msddr = msddr + dkar*dkar
    msddl = msddl + dkal*dkal
    msddc = msddc + dkac*dkac
  end do ! next ik
  msddr = msddr*iorigin
  msddl = msddl*iorigin
  msddc = msddc*iorigin
  ! elapsed time 
  ttime = dble(npos)*dble(nread)*dt
  if (Qregr) then
     msdr(npoints) = msddr
     msdl(npoints) = msddl
     msdc(npoints) = msddc
    elaptime(npoints) = ttime
  endif
  ! write outputfile
  if (Qout2) then 
    write(line,*) ttime,msddr,msddl,msddc
    write(88,'(a)') trim(line)
  endif
end do ! next npos
if (Qout2) close(88)

! *** Linear least square regression 
if (Qregr) then
  write(6,*) '**** CALCULATING LINEAR LEAST SQUARE REGRESSION ***'
  open (91,file=trim(fname5),status='unknown')
  write(line,*) 'Opened ',trim(fname5),' linear regression file'
  write(6,'(a)') trim(line)
  write(91,'(a)') 'Linear least square regression of collective charge'
  write(91,'(a)') '==================================================='
  write(91,*)
  write(91,'(a)') 'right-side difference operator:'
  write(91,'(a)') '------------------------------'
  call linear_regression(npoints,elaptime,msdr) 
  write(91,*)
  write(91,'(a)') 'left-side difference operator:'
  write(91,'(a)') '-----------------------------'
  call linear_regression(npoints,elaptime,msdl)
  write(91,*)
  write(91,'(a)') 'central difference operator:'
  write(91,'(a)') '---------------------------'
  call linear_regression(npoints,elaptime,msdc)
  close(91)  
endif

return
end subroutine

!     ***  Main response_tehory program *********
!
!      written by C. J. F. Solano
!
PROGRAM response_theory

use globalmod

implicit none
! *****local variables
character*144 filename

call getarg(1,filename)

! *** Reading input file
write(6,'(a)') '**** READING INPUT FILE ***'
write(6,'(a)') '==========================='
open(25,file=trim(filename),status="old")
call readinit ! reading input file
close(25)

! *** Amount of charge calculation
if (logQ) then
  write(6,'(a)') '**** AMOUNT OF CHARGE TRANSFER CALCULATION ***'
  write(6,'(a)') '=============================================='
  call Qtransfer
endif

! *** msd for collective charge calculation
if (logmsd) then
  write(6,'(a)') '**** MSD FOR COLLECTIVE CHARGE CALCULATION ***'
  write(6,'(a)') '=============================================='
  call msd
endif

! *** End of run
write(6,*)
write(6,'(a)') 'End of run Ok!!!'

end program
